## About: This create a more saner way of addressing pins in AVR
## Author: Brian Khuu
## e.g. instead of DDRB and PINA, just say DDR.PA1 etc...
# Could use http://www.avrfreaks.net/index.php?name=PNphpBB2&file=viewtopic&t=73829
# But above may be more unweildy to use
import datetime
import sys
import time
from time import gmtime,localtime, strftime


#### FUNCTIONS ####

# update_progress() : Displays or updates a console progress bar
## Accepts a float between 0 and 1. Any int will be converted to a float.
## A value under 0 represents a 'halt'.
## A value at 1 or bigger represents 100%
def update_progress(progress):
    barLength = 20 # Modify this to change the length of the progress bar
    status = ""
    if isinstance(progress, int):
        progress = float(progress)
    if not isinstance(progress, float):
        progress = 0
        status = "error: progress var must be float\r\n"
    if progress < 0:
        progress = 0
        status = "Halt... \r\n"
    if progress >= 1:
        progress = 1
        status = "Done... \r\n"
    block = int(round(barLength*progress))
    text = "\rPercent: [{0}] {1}% {2}".format( "#"*block + "-"*(barLength-block), progress*100, status)
    sys.stdout.write(text)
    sys.stdout.flush()


#################### MAIN SCRIPT ###################################
#### What is the current date and time ####
print ("load time and date")
dateTimeString = strftime("%a %d-%m-%Y %I:%M%p %Ssec",localtime())
filenameTimeStamp = strftime("%a.%d-%m-%Y.H%I_M%M%p.S%S",localtime())

## Config
version = 'V1.1'
packlistfilename = './AVRAugment_io.h'
portLetters = ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P'] # ATmega128 goes up to G, lets do twice more just in case
pinNumbers = [1,2,3,4,5,6,7,8,9,10,12,13,14,15,16] # maybe AVR will turn to 16bit port numbers? donno, just in case.

# Start progress monitor
totalProgress = len(portLetters)*len(pinNumbers)
currentProgress = 0;
update_progress(0)
sl = [] #stringlist
sl.append( '// Generated @ ' + dateTimeString ) 
sl.append(
"""/*
    TITLE:   AVR Augmentation for io.h
    Purpose: Increase portability, Readability, Save time
    AUTHOR:  Brian Khuu
    Version: """+version+"""

    Assumption:
        That you are using io.h (For avr MCU) and
        that it splits IOs into banks of Port letters each with an word sized of pin numbers
            e.g. ( PA5 -> Port A pin 5 . Is a macro within io.h )
        Port letters Supported: """+', '.join(map(str, portLetters))+"""
        Pin numbers Supported: """+', '.join(map(str, pinNumbers))+"""

    cite: initial inspiration from cinderblock "[CODE] [C] Simplifying PORT and DDR #defines for Portability
           http://www.avrfreaks.net/index.php?name=PNphpBB2&file=viewtopic&t=73829
    cite: "GET" and "SET" macros from David Jahshan\'s game_console Template

    Example blink code for the arduino UNO in C located in bottom of this header file
*/

// These accept letter + number reference only.
//      e.g. for PA5 -> SET_PORT(A5,0xFF)
// ( R_ was a cinderblock suggestion, to allow for nested concat operations) 

#define SET_PORT(PIN,VALUE)		R_SET_PORT(	PIN	,	VALUE	)
#define SET_DDR(PIN,VALUE)		R_SET_DDR(	PIN	,	VALUE	)
#define SET_PIN(PIN,VALUE)		R_SET_PIN(	PIN	,	VALUE	)
#define GET_PORT(PIN)			R_GET_PORT(	PIN	)
#define GET_DDR(PIN)			R_GET_DDR(	PIN	)
#define GET_PIN(PIN)			R_GET_PIN(	PIN	)

#define R_SET_PORT(PIN,VALUE)   SET(	PORT_P##PIN	,	BITMASK_P##PIN	,	VALUE	)
#define R_SET_DDR(PIN,VALUE)    SET(	DDR_P##PIN	,	BITMASK_P##PIN	,	VALUE	)
#define R_SET_PIN(PIN,VALUE)    SET(	PIN_P##PIN	,	BITMASK_P##PIN	,	VALUE	)
#define R_GET_PORT(PIN)			GET(	PORT_P##PIN	,	BITMASK_P##PIN	)
#define R_GET_DDR(PIN)			GET(	DDR_P##PIN	,	BITMASK_P##PIN	)
#define R_GET_PIN(PIN)			GET(	PIN_P##PIN	,	BITMASK_P##PIN	)

//// From David Jahshan's SET and GET macro
// This will not accept letter+number reference, must give PORT and MASK reference
// e.g. SET_REGISTER( PORTA, _BV(PA6), 0xFF);
#define SET(REGISTER,MASK,VALUE)	REGISTER = ((MASK & VALUE) | (REGISTER & ~MASK))
#define GET(REGISTER,MASK)			REGISTER & MASK

// Originally from Steve Karg in http://stackoverflow.com/questions/47981/how-do-you-set-clear-and-toggle-a-single-bit-in-c-c
//Mask
#define SETMASK(VARIABLE,MASK)		VARIABLE = VARIABLE | MASK
#define CLEARMASK(VARIABLE,MASK)	VARIABLE = VARIABLE & ~MASK
#define TOGGLEMASK(VARIABLE,MASK)	VARIABLE = VARIABLE ^ MASK
#define CHECKMASK(VARIABLE,MASK)	VARIABLE & MASK
//Single bit
#define SETBIT(VARIABLE,b)			((VARIABLE) |= (1<<(b)))
#define CLEARBIT(VARIABLE,b)		((VARIABLE) &= ~(1<<(b)))
#define FLIPBIT(VARIABLE,b)			((VARIABLE ^= (1<<(b)))
#define CHECKBIT(VARIABLE,b)		((VARIABLE) & (1<<(b)))


// ~~~~~~~~~~ PORT PIN DDR references ~~~~~~~~~~
""")

for portLetter in portLetters:
    sl.append( '//## PORT:'+portLetter)
    sl.append( '' )
    for pinNumber in pinNumbers:
        #must be in string
        pinNumber = str(pinNumber)
        sl.append( '    //P'+portLetter+pinNumber )
        sl.append( '    #define BITMASK_P'+portLetter+pinNumber+'    _BV(P'+portLetter+pinNumber+')' )
        sl.append( '    #define PORT_P'+portLetter+pinNumber+'       PORT'+portLetter)
        sl.append( '    #define DDR_P'+portLetter+pinNumber+'        DDR'+portLetter )
        sl.append( '    #define PIN_P'+portLetter+pinNumber+'        PIN'+portLetter )
        sl.append( '' )
        currentProgress += 1
        update_progress(currentProgress/totalProgress)
        #time.sleep(0.01)
    sl.append( '' )
sl.append( '' )
sl.append( '' )
sl.append( """// ~~~~~~~~~ EXAMPLE ~~~~~~~~~~~
/*


// Example blink code for Arduino UNO 16MHz
#define F_CPU 16000000UL // For delay.h to function
#include <avr/io.h> // PIN, DDR, PORT defs
#include <util/delay.h> //_delay_ms()
#include \""""+packlistfilename+"""\"
#define LED13 B5 // Only need to change this to swap pins
#define DDR_13	SET_DDR( LED13, 0xFF ) //IN=0x00, OUT=0xFF
#define ON_13	SET_PORT( LED13, 0xFF ) //LOW=0x00, HIGH=0xFF
#define OFF_13	SET_PORT( LED13, 0x00 ) //LOW=0x00, HIGH=0xFF
int main(void)
{
	DDR_13;
	ON_13;
    while(1)
    {
		_delay_ms(2000);
		OFF_13;
		_delay_ms(2000);
		ON_13;
    }
}


*/
""" ) 

#Completed generation
#update_progress(1.0)

#save result
print ("saving file")
ext_file = open(packlistfilename, "w")
for line in sl:
    #print line
    ext_file.write(line+"\n")
ext_file.close()

#display file
print ("displaying generated headerfile")
import subprocess
subprocess.call(['notepad.exe',packlistfilename])
    

